﻿Add-Type -AssemblyName System.speech
Add-Type –assemblyName PresentationFramework
Add-Type -assembly "Microsoft.Office.Interop.Outlook"
Import-Module c:\scripts\majel\OpenWeatherMap\OpenWeatherMap


# Majels Crontab lookalike file
$clockwork_file = "C:\scripts\majel\01-clockwork.csv"

# Load Goodbyes from this file
$goodbye_file = "c:\scripts\majel\05-goodbye.csv"

# Hail Sound
$hail_sound = "C:\Windows\Media\Windows Notify Calendar.wav"

# Logs & Alerts
# Admins Mail
$system_alerts = "max.mustermann@gmx.con"
$system_smtpserver = "smtp.gmx.con"
$system_smtpport = "465"
$system_smtpfrom = "Majel <max.mustermann@gmx.con>"
# Majels Global Logfiles
$global:log_file = "c:\scripts\majel\logfile.txt"


# Personal Open Weather API Key
# get your own key for free at http://openweathermap.org/appid
$Global:OWMAPIKey = '<enter your key here>'
# Get Weather data for this region
$weather_city = 'Marburg, DE'
# Weather warning - warn if temperature is getting to low
$weather_warning = "4.0"


#
#
# Project Majel
# Public Release V0.1
# Majel is a mix from HAL, Master Control Program and Siri
# Majel is inspired by Majel Rodenberry, the first computer voice of Star Trek
# Alpha Release, 2015
#
#




# Send email alert to admin
function admin_email([string]$reciever, [string]$mail_body) {

    $smtp = New-Object System.Net.Mail.Smtpclient($system_smtpserver,$system_smtpport)
    $SMTPMessage = New-Object System.Net.Mail.MailMessage
    $SMTPMessage.to.add($reciever)
    $SMTPmessage.from = $system_smtpfrom
    $SMTPMessage.Subject = "Server Error"
    $SMTPMessage.Body = $mail_body
    $smtp.Send($SMTPMessage)
}


#
#
# Credits to https://pmckibbins.wordpress.com/2013/01/21/powershell-text-to-speech/
function Say {
 
   param (
           [Parameter(Mandatory=$true, ValueFromPipeline=$true)]
           [string] $Text,
           [switch] $Async=$false
         )
 
   [Reflection.Assembly]::LoadWithPartialName('System.Speech') | Out-Null
   $object = New-Object System.Speech.Synthesis.SpeechSynthesizer
   $object.Rate = 0
   $object.Volume = 70
   
   if ($Async) {
       $object.SpeakAsync($Text)
   } else {
       $object.Speak($Text)
   }
   log ("Told: " + $Text)
   $object.Dispose()   
}

#
#
# Do a specific sound
function hail {
    $_MediaPlayer = New-Object System.Windows.Media.MediaPlayer 
    $_MediaPlayer.Open($hail_sound)
    $_MediaPlayer.Volume = 1.0
    $_MediaPlayer.Play()
    log ("Hailed")
    start-sleep 3
}

#
#
# Write Logfile
function log ($logcontent) {
    $log_date = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
    add-content -path $log_file -Value ($log_date + ">" + $logcontent)
}



#
#
# Checks the calendar
function check_calendar {

    # time = Today
    $time = Get-Date -Format "MM/dd/yyyy"    

    # Initialize the Calendar_Array (todays calendar items are stored here)
    $Calendar_Array = @()

    # Update: just mapibox1 is used
    for ($mapibox=1; $mapibox -le 1; $mapibox=$mapibox+1)
    {
        # $mapibox = 1: Exchange Default Account
        # if you've got more than 1 Exchange Account, increase -le 1 
        
        ## Connect to Outlook
        # Credits to https://sqlnotesfromtheunderground.wordpress.com/2014/09/06/by-example-powershell-commands-for-outlook/
 
        # Outlook Connection
        $Outlook = New-Object -ComObject Outlook.Application
             
        ## Connect to Calendar 
        $OutlookCalendar = $Outlook.session.Folders.Item($mapibox).Folders("Kalender")
    
        foreach ($Entry in $OutlookCalendar.Items)
        {
            # Convert date
            $Item_Candidat =  [string]$Entry.Start -replace "/","."            
            $Item_Candidat_Date = $Item_Candidat.Substring(0,10)
            $Item_Candidat_Time = $Item_Candidat.Substring(11,5)

            # if something is on the calendar today, put it in $Calendar_Array
            if ($time -eq $Item_Candidat_Date) {
            
                # Calendar_Entry wird jetzt zusammengesetzt
                $Calendar_Entry = $Entry.Subject + ". "

                # Check for location
                if ($Entry.Location -ne "") {
                    $Calendar_Entry = $Calendar_Entry + "Der Termin findet hier statt: " + $Entry.Location + ". "
                }

                # Check for full time appointment            
                if (($Item_Candidat_Time -eq "00:00") -and ($Entry.Duration -eq "1440")) {
                    $Calendar_Entry = $Calendar_Entry + "Der Termin ist ganztägig. "
                }
            
                # if not full time, tell start time and duration
                else {
                    $Calendar_Entry = $Calendar_Entry + "Los geht es um " + $Item_Candidat_Time + ", das Ganze dauert " + $Entry.Duration + " Minuten. "
                }

                # put the whole entry in Calendar Array
                $Calendar_Array += $Calendar_Entry            
            }    
        }


        # Now the full Calendar is parsed
        # Tell the results
    }
    
    hail

    # any Calendar entries?
    if ($Calendar_Array.Count -eq 0) {
        Say ("Für heute steht kein Termin im Kalender.")
    }

    # just one Calendar entry found
    elseif ($Calendar_Array.Count -eq 1) {
        Say ("Für heute steht ein Termin im Kalender.")
        Say ($Calendar_Array[0])
        sleep (1)
        Say ("Das waren schon alle Termine für heute.")
    }
    
    # more than one entry found
    else
    {
        Say ("Heute stehen " + $calendar_array.count + " Termine im Kalender. Ich bitte um Aufmerksamkeit:")
        
        $i = 1;

        foreach ($entry in $Calendar_Array) {
            Say("Termin Nummer " + $i + " lautet:" + $Entry)
            $i ++
        }
        
        Sleep (1)
        
        # extra comment for >= 3 entries found
        if ($Calendar_Array.Count -ge 3) {
            Say ("Das waren alle Termine. Ziemlich viel los heute.")
        
        } else { 
            Say ("Das waren alle Termine für heute.")
        }
    }
    
    # In every case, something was told in this run
    
}


#
#
# Checks if someone from the Outlook Contacts has birthday today
function check_anniversaries {
    
    ## Connect to Outlook
    # Credits to https://sqlnotesfromtheunderground.wordpress.com/2014/09/06/by-example-powershell-commands-for-outlook/

    hail
    Say ("Ich schaue jetzt nach, ob jemand Geburtstag hat. Einen kleinen Moment Geduld bitte.")
 
    # Outlook Connection
    $Outlook = New-Object -ComObject Outlook.Application    
    
    # Read Contacts
    $OutlookContacts = $Outlook.session.Folders.Item(1).Folders("Kontakte").items
    
    $BirthdayInfo=$OutlookContacts | Select-Object FullName,Birthday | Where { $_.Birthday –ne ([datetime]”1/1/4501”) }
    
    $time = Get-Date -Format "MM.dd"   
    
    $told_something = 0
        
    Foreach ($Entry in $BirthdayInfo)
    {
        $born = [string]$entry.Birthday
            
        if (($born -ne "") -and ($born.Length -ge 10)) {
            $born_sub = $born.Substring(0,10)
            $born_monthday = $born_sub.substring(0,5)
            $born_monthday = $born_monthday -replace "/","."            
            $born_year = $born_sub.substring(6,4)
        
            if ($born_monthday -eq $time) {

                $timeStr = Get-Date -Format "yyyy"
                $timeint = $timestr -as [int]
                $timeCandidat = $born_year -as [int]
                $Years = $timeInt - $timeCandidat 
                Say ( "Ich habe ein Geburtstagskind entdeckt. Heute wird " + $entry.FullName + " " + $Years + " Jahre alt.")
                $told_something = $told_something + 1
            }
        }
     
    }

    if ($told_something -eq 0) {Say ("Für heute steht kein Geburtstag im Kalender.")}
}

#
#
# Checks for tasks to be done
function check_tasks {
    
    ## Connect to Outlook
    # Credits to https://sqlnotesfromtheunderground.wordpress.com/2014/09/06/by-example-powershell-commands-for-outlook/

    hail
    Say ("Zu guter Letzt werfe ich noch einen Blick auf die Aufgaben-Liste. Einen kleinen Moment Geduld bitte.")
 
    # Outlook Connection
    $Outlook = New-Object -ComObject Outlook.Application
    # $OutlookContacts = $Outlook.session.GetDefaultFolder(10)

    # Read Contacts
    $OutlookTasks = $Outlook.session.GetDefaultFolder(13).items

    $jobcounter = 0

    Foreach ($job in $OutlookTasks)
    {
        if ($job.Status -eq 0) {
            $jobcounter ++                       
        }
    }

    if ($jobcounter -ne 0)
    {             
     
        #1 Job to do
        if ($jobcounter -eq 1) {
            Say ("Die einzige Aufgabe lautet: ")

            Foreach ($job in $OutlookTasks) {
                if ($job.Status -eq 0) { Say ($job.subject)}
            }
        }
        #more than 1 Job to do
        else
        {
            Say ("Es sind " + $jobcounter + " Einträge auf der Liste.")
            $jobcounter = 0;            
            foreach ($job in $OutlookTasks) {
                if ($job.status -eq 0) {
                    $jobcounter ++
                    Say ("Aufgabe " + $jobcounter + " lautet: " + $job.subject)
                }
            }            
        }
        
    }
    else
    {
        say ("Glückwunsch. Es stehen keine unerledigen Aufgaben auf der Liste.")
    }

}

 

# 
# 
# Check Wheater
function check_weather {

   param (
           [Parameter(Mandatory=$true, ValueFromPipeline=$true)]
           [string] $Type           
         )    
     
    #
    # $Type:
    # full = checks actual temperature + max and min
    # night = checks just min temperature and gives a hint if it's getting to cold

    # Credits to http://www.powershelladmin.com/wiki/Get-Weather_cmdlet_for_PowerShell,_using_the_OpenWeatherMap_API#Introductory_Get-OWMWeather_Example
    # 
     
    $result = $weather_city | Get-OWMWeather -Object 
      
    $TempAktuell = $result.main.temp[0] - 273.15       
    $TempMin = $result.main.temp_min[0] - 273.15  
    $TempMax = $result.main.temp_max[0] - 273.15

     
    $TempMinRound = [System.Math]::Round([double] $TempMin, 1)
    $TempMaxRound = [System.Math]::Round([double] $TempMax, 1)
    $TempAktuellRound = [System.Math]::Round([double] $TempAktuell, 1)

    if ($TempAktuell -eq "-273,15") {
        log ("Error: Could not retrieve any weather data")
        admin_email $system_alerts "Could not retrieve any weather data"
    } else 
    
    {            
        # full temperature reporting
        if ($Type -eq 'full') {        
            # Say Min, Max and Actual Temperature
            hail
            Say ("Jetzt: der Wetterbericht")
            sleep (1)
            if ($TempMinRound -eq $TempMaxRound) {$out = "Die Außentemperatur liegt aktuell bei " + $TempAktuellRound + " Grad."}
            else {$out = "Die Außentemperatur liegt aktuell zwischen " + $TempMinRound + " und " + $TempMaxRound + " Grad. Jetzt haben wir " + $TempAktuellRound + " Grad."}

            $out = $out.Replace(".",",")
            Say ($out)



            # if it's getting to cold, give a hint
            if ($TempAktuellRound -le $weather_warning) {
                sleep (1)
                Say ("Bitte Vorsicht auf der Straße. Es könnte glatt sein.")
            }

            if ($TempMinRound -le $weather_warning) {
                sleep (1)
                Say ("Ach ja, und denkt bitte daran, heute Abend die Fenster zu schließen. Es wird heute Nacht kalt.")
            }
        }


        # just check if it's getting to cold at night
        if ($Type -eq 'night') {            
            # if it's getting to cold, give a hint            
            if ($TempMinRound -le $weather_warning) {
                hail
                $out = "Kurze Durchsage in Sachen Temperatur: heute Abend wird es " + $TempMinRound + " Grad kalt. Ich hoffe, die Fenster sind alle zu."
                $out = $out.Replace(".",",")
                Say ($out)
            }
        }
        # in every case, something was told
        
    }
}


#
#
# Say Goodbye
function goodbye {

    $goodbye = Get-Content $goodbye_file
    $randomQuote = $goodbye | Get-Random -Count 1 
    hail
    Say ($randomQuote)

}

#
#
# Restart server
function restart {
       
    log ("Restarting Server")
    Restart-Computer -Force

}

#
#
# Check the clock
function check_the_clock {

    # Check the clock
    $clock = Get-Date -Format "HH:mm"
    $day = (Get-Date).dayofweek

    $clockitems = Import-CSV $clockwork_file  -Encoding UTF7 -delimiter ";" -Header Day,Time,Text

    $clockworkitems.day
        
    ForEach ($clockitem_candidat in $clockitems)
    {
        if (($clockitem_candidat.Day -eq $day) -and ($clockitem_candidat.time -eq $clock)) 
        {
                
            # Parse for $anniversary_check
            if ($clockitem_candidat.text -like '$anniversary_check')
            {
                check_anniversaries
            }
                
            # Parse for $goodbye - send the children to bed
            elseif ($clockitem_candidat.text -like '$goodbye')
            {
                goodbye
            }

            # Parse for $restart - and restart the server
            elseif ($clockitem_candidat.text -like '$restart')
            {
                restart
            }
            
            # Parse for $check_calendar - look out for appointments
            elseif ($clockitem_candidat.text -like '$check_calendar')
            {
                check_calendar
            }

            # Parse for $check_tasks - look out for todos
            elseif ($clockitem_candidat.text -like '$check_tasks')
            {
                check_tasks
            }

            # Parse for $check_weather - say something about temperatures
            elseif ($clockitem_candidat.text -like '$check_weather("full")')
            {
                check_weather("full")
            }
            
            # Parse for $check_weather - say something about temperatures
            elseif ($clockitem_candidat.text -like '$check_weather("night")')
            {
                check_weather("night")
            }

            
            # Parse for $briefing - gives the full obersight
            elseif ($clockitem_candidat.text -like '$briefing')
            {
                briefing
            }

                
            else {                
                hail
                Say ("Es ist " + $clock + " Uhr.")
                if ($clockitem_candidat.Text -ne "") {Say ($clockitem_candidat.Text)}                    
            }
                
            
        }
    }
}

#
#
# Briefing
function Briefing {
    
    hail
    Say ("Ich bitte um Aufmerksamkeit: Es folgt die Tagesübersicht.")

    check_weather("full")

    # Check the calendar entries
    check_calendar

    # Now start checking for anniversaries
    check_anniversaries

    # New check the todo list
    check_tasks

    
}



# Starting

# Check for time-based events
check_the_clock
